﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace SampleApplication
{
    public partial class Form1 : Form
    {
        internal const float PointSize = 7.5f;

        private LayerOverlay pointOverlay;

        private LayerOverlay thirdOverlay;

        private Feature leftPoint;

        private HighlightingPointStyle pointStyle;

        private Point mouseDownPoint;

        public Form1()
        {
            InitializeComponent();

            Application.ThreadException += OnThreadException;
            AppDomain.CurrentDomain.UnhandledException += OnUnhandledException;

            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.SimpleColors.White);
            winformsMap1.CurrentExtent = new RectangleShape(-85, 90.13, 87.28, -48.04);

            InitializeAreaLayer();
            InitializeLayer();
            InitializeThirdLayer();

            winformsMap1.Refresh();
        }

        private void OnUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            this.timer1.Enabled = false;
        }

        private void OnThreadException(object sender, ThreadExceptionEventArgs e)
        {
            this.timer1.Enabled = false;
        }

        private void InitializeAreaLayer()
        {
            var overlay = new LayerOverlay();
            var layer = new InMemoryFeatureLayer();
            ZoomLevel level = layer.ZoomLevelSet.ZoomLevel01;
            level.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.SimpleColors.Red));
            level.DefaultAreaStyle = new AreaStyle(new GeoSolidBrush(GeoColor.SimpleColors.Yellow));
            level.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            var shape = new EllipseShape(new PointShape(0, 0), 20, 20);
            layer.InternalFeatures.Add(new Feature(shape));
            layer.BuildIndex();
            layer.DrawingQuality = DrawingQuality.HighSpeed;
            overlay.Layers.Add(layer);
            this.winformsMap1.Overlays.Add(overlay);
        }

        private void InitializeLayer()
        {
            this.pointOverlay = new LayerOverlay();
            var layer = new InMemoryFeatureLayer();
            this.leftPoint = AddPoint(layer, 0, 0);
            AddPoint(layer, 1, 0);
            ZoomLevel level = layer.ZoomLevelSet.ZoomLevel01;
            level.DefaultPointStyle = (this.pointStyle = new HighlightingPointStyle(this.leftPoint));
            level.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            AddShapes(layer, 200, 0, 0, 40, (x, y) => new PointShape(x, y));

            layer.DrawingQuality = DrawingQuality.HighSpeed;
            layer.BuildIndex();
            this.pointOverlay.Layers.Add(layer);
            this.winformsMap1.Overlays.Add(this.pointOverlay);
        }

        private static void AddShapes(InMemoryFeatureLayer layer, int columnsCount, double centerX, double centerY, double width, Func<double, double, BaseShape> shapeFactory)
        {
            for (int x = 0; x <= columnsCount; x++)
            {
                for (int y = 0; y <= columnsCount; y++)
                {
                    BaseShape shape = shapeFactory(
                        centerX + width * ((double)x / columnsCount - 0.5),
                        centerY + width * ((double)y / columnsCount - 0.5));
                    var feature = new Feature(shape);
                    layer.InternalFeatures.Add(feature);
                }
            }
        }

        private static Feature AddPoint(InMemoryFeatureLayer layer, double x, double y)
        {
            var shape = new PointShape(x, y);
            var feature = new Feature(shape);
            layer.InternalFeatures.Add(feature);
            return feature;
        }

        private void InitializeThirdLayer()
        {
            this.thirdOverlay = new LayerOverlay();
            /*var layer = new InMemoryFeatureLayer();
            ZoomLevel level = layer.ZoomLevelSet.ZoomLevel01;
            //level.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.SimpleColors.Red));
            //level.DefaultAreaStyle = new AreaStyle(new GeoSolidBrush(GeoColor.SimpleColors.Blue));
            var areaStyle = new AreaStyle(new GeoPen(GeoColor.StandardColors.Black), new GeoSolidBrush(GeoColor.StandardColors.LightGoldenrodYellow));
            level.DefaultTextStyle = TextStyles.CreateMaskTextStyle(
                TextColumnName,
                new GeoFont("Arial", 10),
                new GeoSolidBrush(GeoColor.SimpleColors.Black),
                areaStyle,
                -30,
                20);
            level.DefaultTextStyle.PointPlacement = PointPlacement.UpperCenter;
            level.DefaultTextStyle.RequiredColumnNames.Add(TextColumnName);
            level.DefaultTextStyle.TextColumnName = TextColumnName;
            level.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //layer.DrawingQuality = DrawingQuality.HighSpeed;
            this.thirdOverlay.Layers.Add(layer);*/
            this.winformsMap1.Overlays.Add(this.thirdOverlay);
        }

        private const string TextColumnName = "Text";

        private void button2_Click(object sender, EventArgs e)
        {
            MeasureTime(
                () =>
                {
                    this.pointStyle.IsHighlightActive = !this.pointStyle.IsHighlightActive;
                    /*this.pointOverlay.Lock.EnterWriteLock();
                    Application.DoEvents();
                    this.winformsMap1.Refresh(this.pointOverlay);
                    Application.DoEvents();
                    this.winformsMap1.Refresh(this.pointOverlay);
                    Thread.Sleep(500);
                    Application.DoEvents();
                    this.pointOverlay.Lock.ExitWriteLock();
                    Application.DoEvents();
                    Thread.Sleep(1000);*/
                    this.winformsMap1.Refresh(this.pointOverlay);
                });
        }

        private async void MeasureTime(Action action)
        {
            Stopwatch watch = Stopwatch.StartNew();
            action();
            watch.Stop();
            this.label1.Text = string.Format("Elapsed time: {0} s", (double)watch.ElapsedMilliseconds / 1000);
            this.label1.Left = this.panel1.Width - 8 - this.label1.Width;
            this.label1.Visible = true;
            for (int i = 1; i <= 3; i++)
            {
                await Task.Delay(500);
                this.label1.Visible = false;
                await Task.Delay(500);
                this.label1.Visible = true;
            }
        }

        private void button1_Click(object sender, EventArgs e)
        {
            MeasureTime(
                () =>
                {
                    this.winformsMap1.Refresh(this.thirdOverlay);
                    this.winformsMap1.Refresh(this.pointOverlay);
                });
        }

        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            Console.WriteLine("MouseMove");
            /*PointShape point = ExtentHelper.ToWorldCoordinate(
                this.winformsMap1.CurrentExtent, e.X, e.Y, this.winformsMap1.Width, this.winformsMap1.Height);
            var layer = (InMemoryFeatureLayer) this.thirdOverlay.Layers.First();
            this.thirdOverlay.Lock.EnterWriteLock();
            try
            {
                layer.Open();
                layer.Clear();
                AddShapes(layer, 10, point.X, point.Y, 10, (x, y) => new EllipseShape(new PointShape(x, y), 0.4));
                AddPoint(layer, point.X, point.Y).ColumnValues[TextColumnName] = "Some text";
                layer.Close();
            }
            finally
            {
                this.thirdOverlay.Lock.ExitWriteLock();
            }

            this.winformsMap1.Refresh(this.thirdOverlay);*/
        }

        private void winformsMap1_MouseDown(object sender, MouseEventArgs e)
        {
            this.mouseDownPoint = e.Location;
        }

        private void winformsMap1_MouseUp(object sender, MouseEventArgs e)
        {
            if (!Equals(e.Location, this.mouseDownPoint))
            {
                return;
            }

            this.winformsMap1.Refresh(this.pointOverlay);
        }

        private void winformsMap1_MouseMove_1(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.None)
            {
                this.winformsMap1.Refresh(this.thirdOverlay);
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            this.timer1.Enabled = this.checkBox1.Checked;
        }

        private async void timer1_Tick(object sender, EventArgs e)
        {
            this.winformsMap1.Refresh(this.pointOverlay);
            await Task.Delay(5);
            this.winformsMap1.Refresh(this.thirdOverlay);
        }
    }
}
